"""
This service module provides the API for searchers and engines.
"""

from relevance.service import Service
from relevance.settings import SettingsFactory
from relevance.manager import ResourceManager
from relevance.query import Search
from relevance.query import QueryParser
from relevance.engine import Engine
from relevance.engine import EngineFactory


# Initialize the service
service = Service.instance('search')  # pylint: disable=invalid-name
manager = service.data.manager = ResourceManager(Engine)  # pylint: disable=invalid-name


@service.before_first_request
def _bootstrap():
    """
    Bootstrap the service.
    """
    factory = EngineFactory()
    loader = SettingsFactory(['/etc/relevance', './etc'])
    settings = service.data.settings = loader.load('search')

    engines = settings.get('engines', {}, data_type=dict)
    engines = factory.load_all_engines(engines)

    for item in engines.values():
        item.start()

    manager.save_all(engines)


@service.route('/<engine_name>', methods=['GET'])
def do_search(engine_name: str):
    """
    Perform a search request.

    :Method: GET
    :Path: /engines/`engine_name`
    :Query Arguments: **q** - the search request to execute. See :mod:`relevance.query`.
    :param engine_name: the name of the engine to query.

    :Statuses: `200` on success.
    :Response:
        .. code-block:: json

            {
                "search": {
                    "query": "foo==\"bar\"",
                },
                "results": [
                    {"text": "my test document"},
                    ...
                ],
                "facets": {
                    "foo": {
                        "bar": 42,
                        "baz": 12
                    }
                },
                "time": 0.0432,
                "count": 143
            }
    """
    engine: Engine = manager.resolve(engine_name)  # type: ignore

    query = service.request.args.get('q', '')
    parser = QueryParser()
    search: Search = parser.loads(query)  # type: ignore
    result_set = engine.search(search)

    results = []
    for row in result_set:
        item = {
            '_id': row.uid,
            '_score': row.score,
            '_schema': row.schema,
            '_type': row.doc_type,
        }
        item.update(row)
        results.append(item)

    result = {
        'search': search.to_dict(),
        'results': results,
        'time': result_set.time,
        'count': result_set.total_count,
    }

    if result_set.facets is not None:
        result.update({
            'facets': result_set.facets,
        })

    return service.result(200, result)


@service.route('/<engine_name>/mapping', methods=['GET'])
def get_mapping_doc_types(engine_name: str):
    """
    Get a list of available doc types.

    :Method: GET
    :Path: /engines/`engine_name`/mapping
    :param engine_name: the name of the engine to query.

    :Statuses: `200` on success.
    :Response:
        .. code-block:: json

            ["doc_type1", "doc_type2", ...]
    """
    engine: Engine = manager.resolve(engine_name)  # type: ignore
    result = engine.get_doc_types()
    return service.result(200, result)


@service.route('/<engine_name>/mapping/<doc_type>', methods=['GET'])
def get_mapping(engine_name: str, doc_type: str):
    """
    Get the mapping for a specific document type.

    :Method: GET
    :Path: /engines/`engine_name`/mapping/`doc_type`
    :param engine_name: the name of the engine to query.
    :param doc_type: the name of the document type to obtain mapping from.

    :Statuses: `200` on success.
    :Response:
        .. code-block:: json

            {
                "field_name": {
                    "data_type": "builtins.str",
                    "name": "field_name",
                    "options": {
                        "indexed": "not_analyzed"
                    },
                    "stored": True
                },
                ...
            }
    """
    engine: Engine = manager.resolve(engine_name)  # type: ignore
    result = engine.get_mapping(doc_type).to_dict()
    return service.result(200, result)


@service.after_request
def _service_cors(response):
    """
    Adds the CORS headers to all responses so that the API can be used directly from
    a web browser.
    """
    headers = {
        'Access-Control-Allow-Origin': service.request.headers.get('Origin', '*'),
        'Access-Control-Allow-Credentials': 'true',
        'Access-Control-Allow-Methods': 'POST, OPTIONS, GET, PUT, DELETE, HEAD, TRACE',
        'Access-Control-Allow-Headers': service.request.headers.get(
            'Access-Control-Request-Headers', 'Authorization'
        ),
    }

    for key, value in headers.items():
        response.headers[key] = value

    if service.debug:
        response.headers['Access-Control-Max-Age'] = '1'

    return response


if __name__ == '__main__':
    import os
    service.run(port=int(os.getenv('PORT', 55345)), debug=True)



#!/usr/bin/python3

# pylint: disable-all

import distutils
from setuptools import setup
from setuptools import find_packages

import relevance as main


class CleanCommand(distutils.cmd.Command):  # type: ignore
    description = 'clean the directory from build files'
    user_options = []  # type: ignore

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import os
        from shutil import rmtree
        dirs = [
            '.mypy_cache/',
            'docs/apidoc',
            'build/', 'dist/', '{0}.egg-info/'.format(main.__name__), '.eggs',
        ]
        for dir in dirs:
            rmtree(dir, ignore_errors=True)
        for root, subdirs, files in os.walk('.'):
            rmtree('{0}/__pycache__/'.format(root), ignore_errors=True)


class BuildApidocCommand(distutils.cmd.Command):  # type: ignore
    description = 'build the api documentation'
    user_options = []  # type: ignore

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import sys
        from sphinx.apidoc import main as apidoc
        if apidoc(['setup.py', '-f', '-e', '-o', 'docs/apidoc', main.__name__]) > 0:
            sys.exit(1)


class BuildSphinxCommand(distutils.cmd.Command):  # type: ignore
    description = 'build the Sphinx documentation'
    user_options = []  # type: ignore

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import sys
        from sphinx import main as sphinx
        if sphinx(['setup.py', 'docs/', 'build/docs/']) > 0:
            sys.exit(1)


class ValidateCommand(distutils.cmd.Command):  # type: ignore
    description = 'check source files for errors and warnings'
    user_options = []  # type: ignore

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import os, sys

        if os.fork() == 0:
            from mypy.main import main as mypy
            mypy('', args=[main.__name__])
            return
        _, ex_mypy = os.wait()

        if os.fork() == 0:
            from pylint.lint import Run as pylint
            pylint([main.__name__])
            return
        _, ex_lint = os.wait()

        if ex_lint + ex_mypy > 0:
            sys.exit(1)


setup(
    name=main.__name__,
    version=main.__version__,

    description=main.__doc__.split('\n')[1].strip(),
    long_description=main.__doc__.strip(),
    url='http://www.relevance.io',
    author='OverrideLogic',
    author_email='info@overridelogic.com',
    maintainer=main.__author__,
    maintainer_email=main.__author_email__,

    license=main.__license__,
    platforms=['any'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search'
    ],

    packages=find_packages(exclude=['tests', 'tests.*']),
    provides=[main.__name__],

    cmdclass={
        'clean': CleanCommand,
        'build_apidoc': BuildApidocCommand,
        'build_sphinx': BuildSphinxCommand,
        'validate': ValidateCommand,
    },

    python_requires='>=3.5',
    setup_requires=[
        'Sphinx>=1.6.5',
        'pylint>=1.7.4',
        'mypy>=0.550',
    ],
    tests_require=[],
    test_suite='tests',

    install_requires=[
        'Werkzeug>=0.12.2',
        'Flask>=0.12',
        'anyconfig>=0.9.1',
        'PyYAML>=3.12',
        'Twisted>=17.9.0',
        'python-dateutil>=2.6.1',
        'requests>=2.9.1',
    ],

    entry_points={
        'console_scripts': [
            'relevance=relevance.utils.cli:main',
            'relevance-server=relevance.utils.cli:server',
            'relevance-search=relevance.utils.cli:search',
        ],
    },

    data_files=[
        ('/etc/relevance', [
            'etc/search.json-example',
        ]),
    ],
)
